/*
 * cexeclp.c
 *
 * Copyright (C) February, 1998 Sergei Viznyuk <sergei@phystech.com>
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 */

#include <stdarg.h>
#include "cexeclib.h"

/*****************************************************************************/
int cexeclp(int (*cdialog)(int [3][2]),const char *filename,const char *argc,...)
{
  va_list ap;
  int i,j,sv[3][2];
  errno = 0;
  for (i=0;i<3;i++)
    if ( pipe(sv[i]) == -1 )
      {
	for (j=0;j<i;j++)
	  {
	    close(sv[j][0]);
	    close(sv[j][1]);
	  }
        return -1;
      }
  i = fork();
  if ( i > 0 )
    {
      int status;
      close(sv[0][0]);
      close(sv[1][1]);
      close(sv[2][1]);
      j=cdialog(sv);
      close(sv[0][1]);
      close(sv[1][0]);
      close(sv[2][0]);
      if ( j ) return j;
      waitpid(i,&status,WUNTRACED);
      if ( WIFEXITED(status) ) return WEXITSTATUS(status);
      if ( WIFSIGNALED(status) )
        {
          errno = EINTR;
          return WTERMSIG(status);
        }
      if ( WIFSTOPPED(status) )
        {
          errno = EINTR;
          return WSTOPSIG(status);
        }
      return 0;
    }
  else
    if ( i == 0 )
      {
        if ( dup2(sv[0][0],STDIN_FILENO) == -1 ) exit(-1);
        if ( dup2(sv[1][1],STDOUT_FILENO) == -1 ) exit(-1);
        if ( dup2(sv[2][1],STDERR_FILENO) == -1 ) exit(-1);
	close(sv[0][1]);
	close(sv[1][0]);
	close(sv[2][0]);
        va_start(ap,argc);
        exit(execlp(filename,argc,ap,NULL));
      }
  close(sv[0][0]);
  close(sv[0][1]);
  close(sv[1][0]);
  close(sv[1][1]);
  close(sv[2][0]);
  close(sv[2][1]);
  return i;
}
